; Dots 2
; by Omegamatrix
;



      processor 6502

VSYNC   =  $00
VBLANK  =  $01
WSYNC   =  $02
NUSIZ0  =  $04
NUSIZ1  =  $05
COLUP0  =  $06
COLUP1  =  $07
COLUPF  =  $08
PF0     =  $0D
PF1     =  $0E
PF2     =  $0F
RESP0   =  $10
RESP1   =  $11
RESM0   =  $12
RESM1   =  $13
RESBL   =  $14
GRP0    =  $1B
GRP1    =  $1C
ENAM0   =  $1D
ENAM1   =  $1E
ENABL   =  $1F
HMP0    =  $20
HMP1    =  $21
HMM0    =  $22
HMM1    =  $23
HMBL    =  $24

HMOVE   =  $2A
HMCLR   =  $2B


INTIM   =  $0284
TIM64T  =  $0296

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;      USER CONSTANTS
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;


SHAPE_DOTS   = $33

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;      RIOT RAM
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

       SEG.U RIOT_RAM
       ORG $80
colPtr   ds 2

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;      MAIN PROGRAM
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
       SEG CODE
       ORG $F000

START:
    cld
    ldx    #0
    txa
.loopClear:
    dex
    txs
    pha
    bne    .loopClear



    lda    #$13   ; 3 copies close
    sta    NUSIZ0
    sta    NUSIZ1

    lda   #<ColTab
    sta   colPtr
    lda   #>ColTab
    sta   colPtr+1



;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Vsync
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

MainLoop:
    lda    #$0E
.loopVsync:
    sta    WSYNC
;---------------------------------------
    sta    VSYNC
    lsr
    bne    .loopVsync
    lda    #46
    sta    TIM64T


    ldx   #2   ; M0
    lda   #85
    jsr   HorizPositioning
    inx        ; M1
    lda   #89
    jsr   HorizPositioning

    sta    WSYNC
;---------------------------------------
    sta    HMOVE


.waitVblank:
    lda    INTIM
    bne    .waitVblank
    sta    WSYNC
;---------------------------------------
    sta    VBLANK



;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; top blank lines
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

    ldy    #44
.loopTop:
    sta    WSYNC
;---------------------------------------
    dey
    bne    .loopTop

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; yellow playfield blocks
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

    ldy   #7
.loopBlocks
    sta   WSYNC
;---------------------------------------
    lda   #$1A
    sta   COLUPF
    lda   #$AA
    sta   PF0
    sta   PF2
    lda   #$55
    sta   PF1
    dey
    bpl   .loopBlocks

    sta    WSYNC
;---------------------------------------
    iny
    sty    PF0
    sty    PF1
    sty    PF2
    jmp    DrawDigit


       ORG $F100


DrawDigit:

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; top dash
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;


    dec    $2E
    dec    $2E
    dec    $2E
    nop
    nop
    nop
    nop
    nop



    lda    #SHAPE_DOTS
    ldx    #7
    ldy    ColTab,X
    sty    COLUPF
    sta    GRP0
    asl
    sta    GRP1
    lda    #$8A
    stx    ENAM0
    sta    COLUP0
    stx    ENAM1
    sta    COLUP1
    jmp    StartLoop






.loopDots:
    nop
    nop
    ldy    ColTab,X              ;4  @72
    sty    COLUPF                ;3  @75
StartLoop:
;---------------------------------------
    lda    PF1_LEFT,X            ;4  @3
    sta    PF1                   ;3  @6
    lda    PF2_LEFT,X            ;4  @10
    sta    PF2                   ;3  @13
    lda    PF0_RIGHT,X           ;4  @17
    ldy    PF1_RIGHT,X           ;4  @21

    sta    RESP0                 ;3  @24
    sta    RESP1                 ;3  @27
    sta    RESP0                 ;3  @30
    sta    RESP1                 ;3  @33

    sta    PF0                   ;3  @36
    sty    PF1                   ;3  @39
    lda    #0                    ;2  @41

    sta    RESP0                 ;3  @44
    sta    RESP1                 ;3  @47
    sta    RESP0                 ;3  @50
    sta    RESP1                 ;3  @53
    sta    PF2                   ;3  @56
    sta    PF0                   ;3  @59
    dex                          ;2  @61
    bne    .loopDots             ;2 @63/64








    sta    WSYNC
;---------------------------------------
    lda    #0        ; clear all
    sta    GRP0
    sta    GRP1
    sta    PF1
    sta    PF2
    sta    ENAM0
    sta    ENAM1
    sta    COLUP0
    sta    COLUP1

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; bottom blank lines
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

    ldy    #138
.loopBot:
    sta    WSYNC
;---------------------------------------
    dey
    bne    .loopBot

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; overscan
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

    lda    #2
    sta    WSYNC
;---------------------------------------
    sta    VBLANK
    lda    #23
    sta    TIM64T

.waitOverscan:
    lda    INTIM
    bne    .waitOverscan
    sta    WSYNC
;---------------------------------------
    jmp    MainLoop

       ORG $FF00

HorizPositioning SUBROUTINE
    sec
    sta    WSYNC
;---------------------------------------
.divideBy15
    sbc    #15
    bcs    .divideBy15
    eor    #7
    asl
    asl
    asl
    asl
    sta    HMP0,X
    sta    RESP0,X
    rts


PF1_LEFT:  ; dummy data
PF2_LEFT:
PF0_RIGHT:
PF1_RIGHT:
    .byte $FF
    .byte $FF
    .byte $FF
    .byte $FF
    .byte $FF
    .byte $FF
    .byte $FF
    .byte $FF
    .byte $FF


; values from excel sheet
DonkeyKongPF1_Left:
    .byte $03,$03,$03,$03,$03,$00,$00,$19,$15,$15,$15,$19

DonkeyKongPF2_Left:
    .byte $7A,$5A,$59,$5B,$7A,$00,$00,$4B,$6A,$7A,$5A,$4B


DonkeyKongPF0_Right:
    .byte $90,$D0,$F0,$B0,$90,$00,$00,$90,$50,$70,$50,$90

DonkeyKongPF1_Right:
    .byte $38,$64,$6C,$60,$38,$00,$00,$72,$42,$77,$45,$75



COLOR_PLAYFIELD  = $44

ColTab:
   .byte COLOR_PLAYFIELD
   .byte COLOR_PLAYFIELD+2
   .byte COLOR_PLAYFIELD+4
   .byte COLOR_PLAYFIELD+6
   .byte COLOR_PLAYFIELD+8
   .byte COLOR_PLAYFIELD+6
   .byte COLOR_PLAYFIELD+4
   .byte COLOR_PLAYFIELD+2

       ORG $FFFC

    .word START
    .word START